import { sleep } from './sleep.js';

const DEFAULT_TIMEOUT_SECONDS = 300;
const DEFAULT_MIN_DELAY_SECONDS = 1;
const DEFAULT_MAX_DELAY_SECONDS = 30;

/**
 * Creates an exponential backoff interval strategy.
 *
 * @param minDelay - The minimum delay before the next try in seconds
 * @param maxDelay - The maximum delay before the next try in seconds
 * @returns An exponential backoff generator
 *
 * @internal
 */
function* createExponentialBackoffStrategy(minDelay, maxDelay) {
  if (minDelay < 1 || maxDelay < 1 || minDelay > maxDelay) {
    throw new Error('Waiter: minDelay must be >= 1 and maxDelay must be >= minDelay');
  }
  let attempt = 1;
  const ceiling = Math.log(maxDelay / minDelay) / Math.log(2) + 1;
  const randomInRange = (min, max) => min + Math.random() * (max - min);
  // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
  while (true) {
    if (attempt > ceiling) {
      yield maxDelay;
    } else {
      yield randomInRange(minDelay, minDelay * 2 ** (attempt - 1));
    }
    attempt += 1;
  }
}

/**
 * Tries a specific logic several times until it succeeds, timeouts, or throws an exception.
 *
 * @param retry - The function to retry logic between each interval
 * @param strategy - A generated interval strategy iterator
 * @param timeout - The maximum time elapsed before timeout error
 *
 * @throws An timeout exception or error thrown by the logic being run
 *
 * @internal
 */
const tryAtIntervals = async function (retry, strategy, timeout) {
  if (timeout === void 0) {
    timeout = DEFAULT_TIMEOUT_SECONDS;
  }
  const timeoutTimestamp = Date.now() + timeout * 1000;
  let retryCount = 0;
  while (Date.now() <= timeoutTimestamp) {
    const delay = strategy.next(retryCount += 1).value * 1000;
    // Break if timeout has been reached
    if (timeoutTimestamp <= Date.now() + delay) break;
    // Wait before the next retry
    await sleep(delay);
    // Retry
    const {
      value,
      done
    } = await retry();
    if (done) return value;
  }
  throw new Error(`Timeout after ${timeout}s`);
};

/**
 * Represents the condition to stop waiting for a resource.
 *
 * @public
 */

/**
 * The options to wait until a resource is ready.
 *
 * @public
 */

/**
 * Fetches resource several times until an expected condition is reached, timeouts, or throws an exception.
 *
 * @param stop - The condition to stop waiting
 * @param fetcher - The method to retrieve resource
 * @param request - The resource request options
 * @param options - The retry strategy options
 * @param strategy - An optional custom strategy
 *
 * @returns A promise of resource
 *
 * @public
 */
const waitForResource = function (stop, fetcher, request, options, strategy) {
  if (strategy === void 0) {
    strategy = createExponentialBackoffStrategy(options?.minDelay ?? DEFAULT_MIN_DELAY_SECONDS, options?.maxDelay ?? DEFAULT_MAX_DELAY_SECONDS);
  }
  return tryAtIntervals(async () => {
    const value = await fetcher(request);
    return {
      done: await stop(value),
      value
    };
  }, strategy, options?.timeout);
};

export { createExponentialBackoffStrategy, tryAtIntervals, waitForResource };
