import { getLogger } from '../../internal/logger/index.js';
import { shouldLog, LevelResolver } from '../../internal/logger/level-resolver.js';
import { dumpRequest, dumpResponse } from './http-dumper.js';

/**
 * Mapper of an header entry.
 *
 * @internal
 */

/**
 * HTTP Request with obfuscated secrets.
 *
 * @internal
 */
class ObfuscatedRequest extends Request {
  constructor(request, obfuscate) {
    super(request);
    this.request = request;
    this.obfuscate = obfuscate;
  }
  get headers() {
    return new Headers(Array.from(this.request.headers, this.obfuscate));
  }
  clone() {
    return new ObfuscatedRequest(this.request, this.obfuscate);
  }
}

/**
 * Creates an interceptor to obfuscate the requests.
 *
 * @param obfuscate - The Header entries obfuscator mapper
 * @returns The obfuscated Request
 *
 * @internal
 */
const obfuscateInterceptor = obfuscate => _ref => {
  let {
    request
  } = _ref;
  return new ObfuscatedRequest(request, obfuscate);
};
const identity = _ref2 => {
  let {
    request
  } = _ref2;
  return request;
};

/**
 * Creates an interceptor to log the requests.
 *
 * @param identifier - The request identifier
 * @param obfuscate - The obfuscation interceptor
 * @returns The interceptor
 *
 * @internal
 */
const logRequest = function (identifier, obfuscate) {
  if (obfuscate === void 0) {
    obfuscate = identity;
  }
  return async _ref3 => {
    let {
      request
    } = _ref3;
    if (shouldLog(LevelResolver[getLogger().logLevel], 'debug')) {
      getLogger().debug(`--------------- Scaleway SDK REQUEST ${identifier} ---------------
${await dumpRequest(await obfuscate({
        request
      }))}
---------------------------------------------------------`);
    }
    return request;
  };
};

/**
 * Creates an interceptor to log the responses.
 *
 * @param identifier - The request identifier
 * @returns The interceptor
 *
 * @internal
 */
const logResponse = identifier => async _ref4 => {
  let {
    response
  } = _ref4;
  if (shouldLog(LevelResolver[getLogger().logLevel], 'debug')) {
    getLogger().debug(`--------------- Scaleway SDK RESPONSE ${identifier} ---------------
${await dumpResponse(response)}
---------------------------------------------------------`);
  }
  return response;
};

export { logRequest, logResponse, obfuscateInterceptor };
